function removeAppointmentIdPrefix(id) {
  return id.substring(1);
}

function isStandardAppointment(event) {
  return event.calendar_entry_id.charAt(0) == "a"
}

function createEvent(value) {
    var event = {};
    event.start = parseCanonicalTimeString(value.start_time);
    event.end = parseCanonicalTimeString(value.end_time);
    event.orig_start_time = value.orig_start_time;
    event.allDay = false;
    event.id = value.id;
    event.calendar_entry_id = value.id;
    event.resourceId = value.provider_id;
    event.tooltip = "";
    event.client_is_new = value.client_is_new;
    event.client_email = value.client_email;
    event.client_unsubscribed_appointment_emails = value.client_unsubscribed_appointment_emails;

    event.setup_duration_minutes = value.setup_duration_minutes;
    event.cleanup_duration_minutes = value.cleanup_duration_minutes;

    event.gap_duration_minutes = value.gap_duration_minutes ? value.gap_duration_minutes : 0;
    event.finish_duration_minutes = value.finish_duration_minutes ? value.finish_duration_minutes : 0;

    event.originated_in_schedulista = value.originated_in_schedulista;
    event.override_as_available = value.override_as_available;
    event.readonly_appointment = value.readonly_appointment;

    event.is_recurrence = value.is_recurrence;
    event.is_recurrence_override = value.is_recurrence_override;

    event.is_class = value.is_class;
    event.capacity = value.capacity;
    event.max_capacity = value.max_capacity;

    if (value.client_full_name_with_new) {
        event.tooltip += "<br>" + value.client_full_name_with_new;
    }

    if (value.service_name) {
        event.tooltip += "<br>" + value.service_name;
    }

    if (value.entry_type != "personal") {
        var notes = ""

        if (value.appointment_notes && $.trim(value.appointment_notes) != "") {
            notes += "<b>Appointment Notes:</b><br>" + value.appointment_notes;
        }

        if (value.client_notes && $.trim(value.client_notes) != "") {
            if (notes != "") {
                notes += "<br><br>"
            }
            notes += "<b>Client Notes:</b><br>" + value.client_notes
        }

        if (notes != "") {
            event.notes = notes.replace(/\n/g, "<br>");
            event.tooltip += "<br><br>" + event.notes;
        }
    }

    if (value.entry_type == "personal") {
        if(!value.originated_in_schedulista && globals.PROVIDER_ID != globals.ACTING_PROVIDER_ID) {
            event.title = "Google Calendar Appointment";
            event.tooltip = "";
            event.editable = false;
        } else if (value.appointment_notes == null || $.trim(value.appointment_notes) == "") {
            if(value.originated_in_schedulista) {
                event.title = transl("calendar.personal");
                event.tooltip += "<br>" + transl("calendar.personal");
            } else {
                event.title = "(No title)";
                event.tooltip = "<br>(No title)";
            }
        } else {
            event.title = value.appointment_notes;
            event.tooltip += "<br>" + value.appointment_notes;
        }
    } else {
        if(event.is_class) {
            event.title =  value.capacity + "/" + value.max_capacity;
        } else {
            event.title = value.client_full_name;
        }

        event.title += " - " + value.service_name
    }

    if(event.readonly_appointment == true) {
        event.editable = false;
    }

    if (event.orig_start_time) {
        event.id += "#" + parseCanonicalTimeString(event.orig_start_time).getTime();
    }

    /*
     dark pink: 9f0351
     dark green: 1d651d
     dark bl: 0f109e
     green: #228822
     blue: #1111cc
     orange #ff6600
     red: #ff0000
     purple: #551a8b
     yellow: #fbb818
     pink: #cc0066
     */

    backgroundColor = "";
    if (value.entry_type == "personal") {
        if(value.originated_in_schedulista) {
            backgroundColor = "#f63757"
        } else {
            backgroundColor = "#00cceb"
        }
    } else if (value.entry_type == "client_scheduled") {
        backgroundColor = "#00c18c"
    } else {
        backgroundColor = "#1c7fe3"
    }

    event.borderColor = "white"
    event.backgroundColor = backgroundColor

    return event;
}

function onEventRender(event, element, view) {
    var pixelsPerMinute = (calendarZoom == "30_minute") ? 0.6993055556 : 0.6993055556 * 2;
    var override_as_available = event.override_as_available;
    var duration_minutes = getDatesDeltaInMinutes(event.end, event.start);
    var setup_duration = 0;
    var gap_duration = event.gap_duration_minutes;
    var finish_duration = event.finish_duration_minutes;
    var start_duration = duration_minutes - (setup_duration + gap_duration + finish_duration);

    var gapTopPixels = (setup_duration + start_duration) * pixelsPerMinute;
    var finishTopPixels = (setup_duration + start_duration + gap_duration) * pixelsPerMinute;
    var setupHeightPixels = (setup_duration) * pixelsPerMinute;
    var startHeightPixels = Math.max((start_duration) * pixelsPerMinute - element.find(".fc-event-head").height(), 0);

    if (event.gap_duration_minutes > 0) {
        var minHeight = (setup_duration + start_duration + gap_duration + 15) * pixelsPerMinute;
        element.css("min-height", minHeight);
        element.find(".fc-event-inner").css("min-height", minHeight);
    }

// TODO: fix if start_duration < 0

    var bgcolor = element.find(".fc-event-inner").css("background-color");

// configure setup block
/*
    if (setup_duration > 0) {
        element.find(".fc-event-inner").prepend(
            "<div class=\"fc-event-setup\" style=\"z-index: 999; border-bottom: 1px dashed white; background-color: " + bgcolor + "; width: 100%; height: "
                + setupHeightPixels + "px;\"></div>");
    }
*/

// configure start block
    var fc_event_content = element.find(".fc-event-content");
    fc_event_content.css("background-color", bgcolor);
    fc_event_content.css("overflow", "hidden");
    fc_event_content.css("height", startHeightPixels + "px");

// configure gap background
    var fc_event_bg = element.find(".fc-event-bg");
    fc_event_bg.css("top", gapTopPixels);
    fc_event_bg.css("height", gap_duration * pixelsPerMinute + "px");

    var view = $('#calendar').fullCalendar('getView').name;
    if (finish_duration > 0 && view != "month") {
        // add transparent backgrounds
        element.css("background-color", "transparent");
        element.find(".fc-event-inner").css("background-color", "transparent");

        // configure finish block
        element.find(".fc-event-inner").append(
            "<div class=\"fc-event-finish\" style=\"z-index: 999; background-color: " + bgcolor + "; position: absolute; width: 100%; top: " +
                finishTopPixels + "px; bottom:0;\"></div>");
    }

    if (highlight_event_id != null && highlight_event_id == event.id) {
        element.css("border", "3px solid orange");
        element.animate({
            "borderTopColor" : "white",
            "borderBottomColor" : "white",
            "borderLeftColor" : "white",
            "borderRightColor" : "white"
        }, 6000);
    }
    event.element = element;

    noteHeightPixels = setupHeightPixels + 3;
    element = element.find(".fc-event-inner")
    element = element.append('<div class="calendar-icons-area" style="top: ' + noteHeightPixels + 'px;"></div>');

    e = element.find(".calendar-icons-area");
    e.css("background-color", event.backgroundColor);
    if (event.is_recurrence || event.is_recurrence_override) {
        if (event.is_recurrence) {
            img = e.append('<img class="calendar-recurrence-image" src="images/recurrence-icon.png" />');
        } else if (event.is_recurrence_override) {
            img = e.append('<img class="calendar-recurrence-image" src="images/recurrence-override-icon.png" />');
        }
    }

    if (event.notes) {
        img = element.children(".calendar-icons-area").append('<img class="calendar-notes-image" src="images/notes_icon.png" />');
    }
}

function onEventAfterRender(event, element, view) {
    if (event.client_is_new) {
        var starHtml = "<img class='new-client-star' src='images/new-client-star-small.png' />";

        var regex = new RegExp(RegExp.quote(event.title));
        var timeElem = element.find('.fc-event-time');
        if (timeElem.text().match(regex)) {
            timeElem.html(timeElem.text().replace(regex, starHtml + event.title));
        } else {
            var titleElem = element.find('.fc-event-title');
            titleElem.html(starHtml + titleElem.text());
        }
    }

    if(event.override_as_available) {
        element.find(".fc-event-inner").css("border", "2px dashed");
    }
}

function minutesSinceStartOfDay(date) {
    var minutes = date.getMinutes();
    if(date.getSeconds() >= 30) {
        minutes += 1
    }
    return  minutes + 60 * date.getHours();
}

// TODO: These are not shared by mobile.
function renderDayViewBlocks(unavailableRanges, wday) {
    if(!$('#day-unavailable-blocks').length) {
        $(".fc-agenda-slots").filter(":visible").parent().append("<div id='day-unavailable-blocks' style='position:absolute;z-index:-1;top:0;left:0'></div>");
    }

    var blockDiv = $("#day-unavailable-blocks");
    var pixelsPerMinute = blockDiv.parent().height() / 24.0 / 60.0;

    var xOffset = 60;

    blockDiv.empty();

    var d = $('#calendar').fullCalendar('getDate').getDay();
    $.each(providersOrder, function(index, value) {
        var width = $(".fc-view-resourceDay .fc-col" + index).width();
        $.each(unavailableRanges[value.id][wday], function(index, value) {
            var begin = value.begin;
            var end = value.end;

            blockDiv.append("<div style='background-color: #f1f1f1; position: absolute; z-index: 8; top: 0px; left: 60px; width: 300px; height: 300px;'></div>");
            blockDiv.children(":last").css("top", pixelsPerMinute * begin);
            blockDiv.children(":last").css("left", xOffset);

            blockDiv.children(":last").width(width);
            blockDiv.children(":last").height(Math.ceil((end - begin) * pixelsPerMinute));
        });
        xOffset += width + 1;
    });
}

function renderWeekViewBlocks(unavailableRanges) {
    if(!$('#week-unavailable-blocks').length) {
        $(".fc-view-agendaWeek .fc-agenda-slots").parent().append("<div id='week-unavailable-blocks' style='position:absolute;z-index:-1;top:0;left:0'></div>");
    }

    var pixelsPerMinute = $("#week-unavailable-blocks").parent().height() / 24.0 / 60.0

    var header = [ 'fc-sun', 'fc-mon', 'fc-tue', 'fc-wed', 'fc-thu', 'fc-fri', 'fc-sat']

    var xOffset = 60;

    $("#week-unavailable-blocks").empty();
    for (var i = 0; i < 7; i++) {
        var width = $(".fc-view-agendaWeek ." + header[i]).width();
        $.each(unavailableRanges[i], function(index, value) {
            var begin = value.begin;
            var end = value.end;

            $("#week-unavailable-blocks").append("<div style='background-color: #f1f1f1; position: absolute; z-index: 8; top: 0px; left: 60px; width: 300px; height: 300px;'></div>");
            $("#week-unavailable-blocks").children(":last").css("top", pixelsPerMinute * begin);
            $("#week-unavailable-blocks").children(":last").css("left", xOffset);

            $("#week-unavailable-blocks").children(":last").width(width);
            $("#week-unavailable-blocks").children(":last").height(Math.ceil((end - begin) * pixelsPerMinute));
        });

        xOffset += width + 1;
    }
}

// Returns string based on how far into the day a time is: e.g. 01:15:00 => 1 hour 15 minutes.
function getHoursMinutesString(time) {
    var hoursString;
    var minutesString;
    var daysString;
    var hours = time.getHours();
    var minutes = time.getMinutes();

    var days = Math.floor((time - new Date(0, 0, 0, 0, 0, 0)) / 1000 / 60 / 60 / 24);

    if(days > 0) {
        daysString = days + " " + (days == 1 ? transl('shared.day.one') : transl('shared.day.other'))
    }

    if (hours > 0) {
        hoursString = hours + " " + (hours == 1 ? transl('shared.hour.one') : transl('shared.hour.other'));
    }

    if (minutes > 0) {
        minutesString = minutes + " " + (minutes == 1 ? transl('shared.minute.one') : transl('shared.minute.other'));
    }

    var result = ""

    if(daysString) {
        result += daysString;
    }

    if (hoursString) {
        if(result.length > 0) {
            result += " "
        }
        result += hoursString;
    }

    if(minutesString) {
        if(result.length > 0) {
            result += " "
        }
        result += minutesString;
    }

    if(result == "") {
        result = "0 minutes";
    }

    return result;
}

function getDurationOptions(max_duration, extra_option) {
    var step = 5;
    var startTime = new Date(0, 0, 0, 0, 5, 0);
    var endTime = new Date(0, 0, 0, max_duration, 0, 0);

    var times = [];
    var time = new Date(startTime);
    var minutes = startTime.getMinutes();

    while(time <= endTime) {
        if(extra_option != null) {
            if(extra_option < minutes) {
                times[times.length] = {
                    text: getHoursMinutesString(new Date(0, 0, 0, 0, extra_option, 0)),
                    value: extra_option
                }
                extra_option = null;
            }
        }

        times[times.length] = {
            text: getHoursMinutesString(time),
            value: minutes
        }

        minutes += step;
        time = new Date(time.setMinutes(time.getMinutes() + step));
    }

    if(extra_option) {
        times[times.length] = {
            text: getHoursMinutesString(new Date(0, 0, 0, 0, extra_option, 0)),
            value: extra_option
        }
    }

    return times;
}