// shared.js

String.prototype.capitalize = function() {
    return this.charAt(0).toUpperCase() + this.slice(1);
}

RegExp.quote = function(str) {
    return (str+'').replace(/([.?*+^$[\]\\(){}|-])/g, "\\$1");
};

function generateGuid() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
        var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
        return v.toString(16);
    });
}

function parseCanonicalTimeString(s) {
  return parseDate(s, false);
}

function canonicalToExtraShortTimeString(timeString) {
    var date = parseCanonicalTimeString(timeString);
    return date.toString(transl("shared.javascript.time_formats.extra_short_time")).toLowerCase();
}

function canonicalToTimeString(timeString) {
    var date = parseCanonicalTimeString(timeString);
    return date.toString(transl('shared.javascript.time_formats.short_time'));
}

function getCanonicalTimeString(date) {
    return formatDate(date, "yyyy-MM-dd'T'HH:mm:ss");
}

function addMinutes(date, n) {
    var d = date.clone();
	d.setMinutes(d.getMinutes() + n);
	return d;
}

function addHours(date, n) {
    var d = date.clone();
	d.setHours(d.getHours() + n);
	return d;
}

function beginning_of_day(date) {
    var d = date.clone();
    d.setMinutes(0);
    d.setHours(0);
    d.setSeconds(0);
    return d;
}

function getDatesDeltaInMinutes(firstDate, secondDate) {
    if(!firstDate) {
        return 0;
    }

    return (firstDate.getTime() - secondDate.getTime()) / (1000 * 60);
}

/*
function subtractMinutes(d, n) {
	d.setMinutes(d.getMinutes() - n);
	return d;
}

function subtractHours(d, n) {
	d.setHours(d.getHours() + n);
	return d;
}
*/

function getShortDateString(date) {
    return date.toString("MMM d");
}

function getDateString(date) {
    return date.toString(transl("shared.javascript.time_formats.medium_date"));
}

function getOriginalCanonicalTimeString(timeString, dayDelta, minuteDelta) {
    var result = parseCanonicalTimeString(timeString);
    result.addDays(-dayDelta);
    result.addMinutes(-minuteDelta);
    // Time + offset
    return getCanonicalTimeString(result);
}

function advanceCanonicalTimeStringByMinutes(timeString, minutes) {
    var result = parseCanonicalTimeString(timeString);
    result.setTime(result.getTime() +  1000 * 60 * minutes);
    return getCanonicalTimeString(result);
}

function addParametersToUrl(url, parameters) {
    var retVal = url;
    for (var key in parameters) {
        retVal = jQuery.param.querystring(retVal, key + '=' + parameters[key]);
    }

    return retVal;
}

function ordinal(number) {
  var i = number.toString();
  if ( i.match(/\D/) ) return i;
  if ( i == 3 || i.match(/[^1]3$/) ) return i + transl('shared.ordinals.third');
  if ( i == 2 || i.match(/[^1]2$/) ) return i + transl('shared.ordinals.second');
  if ( i == 1 || i.match(/[^1]1$/) ) return i + transl('shared.ordinals.first');
  return i + transl('shared.ordinals.nth');
}

function pluralize(number, str) {
    if(number == 0 || number > 1) {
        return number + " " + transl(str + ".other")
    } else {
        return number + " " + transl(str + ".one");
    }
}

function minutesToDisplayString(minutes) {
    var hourPart = Math.floor(minutes / 60)

    var hoursString = null;
    if(hourPart > 0) {
        hoursString = pluralize(hourPart, "shared.hour");
    }

    var minutePart = minutes % 60;
    var minutesString = null
    if(minutePart > 0) {
        minutesString = pluralize(minutePart, "shared.minute");
    }

    if(hoursString && minutesString) {
      return hoursString + " " + minutesString;
    } else if(hoursString) {
        return hoursString;
    } else if (minutesString) {
        return minutesString;
    } else {
        return pluralize(0, "shared.minute");
    }
}


$(document).ready(function() {
    // Don't allow users to click submit button multiple times.
    // TODO: Solve this so ajax submits work.
    $("form").not('[data-remote="true"]').submit(function() {
        $('input[type=submit]', this).attr('disabled', 'disabled');
        $(this).submit(function() {
            return false;
        });
        return true;
    });

  // Javascript spinner.
  $('.loadingDiv').html('<img src="/images/ajax-loader.gif" alt="Loading..." />')

  $('.loadingDiv')
    .hide()
    .ajaxStart(function() {
        $('.ajaxData').hide();
        $(this).show();
    })
    .ajaxStop(function() {
        $(this).hide();
        $('.ajaxData').show();
    });
});

var global_messageTimeout = null;

function showStatusMessage(message, timeoutSeconds, onFadeOut) {
    if(global_messageTimeout) {
        clearTimeout(global_messageTimeout);
        global_messageTimeout = null;
    }


    $("#status-text").stop(true).animate({opacity:'100'});

    $("#status-text").html(message);

    $("#status-text").show();

    $("#status-text").css('top', 108);

    $("#status-text").css('left', ($("#left-content").width() / 2) - ($("#status-text").width() / 2) + 3);

    global_messageTimeout = setTimeout(function() {
        $("#status-text").fadeOut(1000, function() {
            if(onFadeOut) {
                onFadeOut();
            }
        });
    }, timeoutSeconds * 1000);
}

// BEGIN Time selection helpers. These probably belong in their own file.
// -----------------------------------------------------------
function populateStartTimeOptions(availableTimes, selectedStartTime) {
    if (availableTimes.length > 0) {
        var startTimeOptions = "";
        $.each(availableTimes, function(index, value) {
            startTimeOptions += '<option ' + ((value.startTime == selectedStartTime) ? 'selected="selected" ' : "")
                    + 'value="' + value.startTime + '">' + canonicalToExtraShortTimeString(value.startTime) + '</option>';
        });
        $('#startTime').html(startTimeOptions);
    } else {
        alert("No times available");
    }
}

function showHideOverlapMessage(appointments) {
    var len = appointments.length;
    var appointment_start = $("#startTime").val();
    var appointment_end = $("#end_time").val();

    for(var i = 0; i < len; ++i) {
        if (appointments[i].startTime < appointment_end &&
                appointments[i].endTime > appointment_start) {
            $("#overlapMessage").html("Overlaps with another appointment (" + appointments[i].serviceName +
                    " at " + canonicalToExtraShortTimeString(appointments[i].startTime) + ").")
            return;
        }
    }
    $("#overlapMessage").html("")
}

// -----------------------------------------------------------
// END Time selection helpers.


// TODO: this function is only used by the backend, so could get moved to a shared backend-specific file.
function attachTip(id, tip) {
    $(id).qtip({
        content: tip,
        style: {
            width: 200,
            padding: 5,
            color: 'black',
            tip: 'bottomLeft',
            fontSize: '12px',
            border: {
                width: 1,
                radius: 1,
                color: 'darkgray'
            }
        },
        show: 'mouseover',
        hide: 'mouseout',
        position: {
            corner: {
                target: 'topMiddle',
                tooltip: 'bottomLeft'
            }
        }

    });
}


function markUnsavedChanges() {
    isDirty = true;
    $("#save_button").removeAttr("disabled");
}

function markDirtyOnValueChange() {
    // Save current value of element
    $(this).data('oldVal', $(this).val());

    // Look for changes in the value
    $(this).bind("propertychange keyup input paste", function(event){
        // If value has changed...
        if ($(this).data('oldVal') != $(this).val()) {
            // Updated stored value
            $(this).data('oldVal', $(this).val());

            // Do action
            markUnsavedChanges();
        }
    });
};

var SchedulistaLoc = SchedulistaLoc || []

function transl(key) {
    var tokens = key.split(".");
    var result;

    var count = SchedulistaLoc.length;
    for(var j =0; j < count; j++) {
        result = SchedulistaLoc[j];

        var len = tokens.length;
        for(var i = 0; i < len; i++) {
            result = result[tokens[i]];
            if(!result) {
                break;
            }
        }

        if(result) {
            break;
        }
    }

    //if(!result && DEVELOPMENT_MODE) {
        //alert("missing translation: " + key)
    //}
    return result
}